/*******************************************************************
file piecewin.c
all piece window manipulation here
*******************************************************************/
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <string.h>
#include "piecewin.h"
#include "gridwin.h"

Window piece_windows[6];
struct piece_rec piece_array[6];
/*******************************************************************
void info(char *text)
provide feedback for user
*******************************************************************/
void info(char *text)
	{
	char clear[] = {"                                                     "};
	extern Display *display;
	extern Window main_window;
	extern GC text_gc;
	XDrawImageString(display,main_window,text_gc,20,620,clear,strlen(clear)); 
	XDrawImageString(display,main_window,text_gc,20,620,text,strlen(text)); 
	}
/*******************************************************************
void draw_pixel(int x, int y, Window window, int piece_num)
place a 3*3 pixel on the appropiate window at location x,y
*******************************************************************/
void draw_pixel(int x, int y, Window window, int piece_num)
	{
	extern Display *display;
	extern GC text_gc;
	extern int shades[8][3][3];
	
	int i,j;
	for (i=0;i<3;i++)
		for (j=0;j<3;j++)
			{
			if (shades[piece_num][i][j])
				XDrawPoint(display,window,text_gc,x+i,y+j);
			}
	};
/*******************************************************************
void display_block_on_piecewin(int x, int y, int piece_num)
draw a block on a piecewindow - 3 blocks to a piece
*******************************************************************/

void display_block_on_piecewin(int x, int y, int piece_num)
	{
	extern int active_piece;
	extern Window piece_windows[6];
	int tempx,tempy,i,j;
	tempx = x * 18 - 18;
	tempy = y * 18;
	for (j=0;j<12;j++)
		for (i=0;i<6;i++)
			draw_pixel(tempx + j * 3,
				   tempy + i * 3,
				   piece_windows[piece_num],
				   piece_num);
	};

/*******************************************************************
void display_piece_on_piecewin(int piece_num)
draw a piece on its correct window - just displays each
block in turn
*******************************************************************/
void display_piece_on_piecewin(int piece_num)
	{
	extern int active_piece;
	extern struct piece_rec piece_array[6];
	extern Display *display;
	extern Window piece_windows[6];	
	int x,y;
 	int i;	
	XClearWindow(display,piece_windows[piece_num]);

	/* calculate some offsets. these should be more
	precise but lack of time ... */
	if (piece_array[piece_num].block[1].x  < 0)
		{
		x = 4;
		}
	else
		x = 1;
	if (piece_array[piece_num].block[1].y < 0)
		{
		y = 4;
		}
	else
		y = 1;

	
	for (i=0;i<3;i++)
		{
		display_block_on_piecewin(x + piece_array[piece_num].block[i].x,
				       y + piece_array[piece_num].block[i].y,piece_num);
  		}
	};


/* open the windows and set the event input flags */
void pieces_init(void)
	{
	extern Display *display;
	
	extern Window piece_windows[6],main_window,gridwindow;
	extern unsigned long foreground,background;
	int i;
	for (i=1;i<6;i++)
		{
		piece_windows[i] = XCreateSimpleWindow(display,main_window,
			(((i-1) * 100)),
			80 +(MAXSIZE*GRID_SIZE),90,90,2,foreground,background);
		XSelectInput(display,piece_windows[i],ButtonPressMask | ExposureMask);
		XMapWindow(display,piece_windows[i]);
		}
	DEBUG("getting pieces\n");
        get_pieces();

	};

/* load in the piece information */
void get_pieces(void)
	{
	extern Display *display;
	extern Window gridwindow;
	extern struct piece_rec piece_array[6];
        FILE* fp;
	int i,j;
        fp = fopen("pieces","r");
	for (i=0;i<6;i++)
		{
                piece_array[i].location.x = 0;
		piece_array[i].location.y = 0;
		piece_array[i].in_use = false;
		switch ( i ){
		case 1	:piece_array[i].block[0].x = 1;
			 piece_array[i].block[0].y = 1;
			 piece_array[i].block[1].x = 3;
			 piece_array[i].block[1].y = 1;
			 piece_array[i].block[2].x = 5;
			 piece_array[i].block[2].y = 1;

			 /* one offset for each quadrant - see gridwin.c */
			 piece_array[i].offset[0].x = 0;
			 piece_array[i].offset[0].y = 0;
			 piece_array[i].offset[1].x = 6;
			 piece_array[i].offset[1].y = 0;
			 piece_array[i].offset[2].x = 6;
			 piece_array[i].offset[2].y = 2;
			 piece_array[i].offset[3].x = 0;
			 piece_array[i].offset[3].y = 2;
                	 break;
		case 2  :piece_array[i].block[0].x = 1;
			 piece_array[i].block[0].y = 1;
			 piece_array[i].block[1].x = 2;
			 piece_array[i].block[1].y = 2;
			 piece_array[i].block[2].x = 1;
			 piece_array[i].block[2].y = 3; 
			 piece_array[i].offset[0].x = 0;
			 piece_array[i].offset[0].y = 0;
			 piece_array[i].offset[1].x = 2;
			 piece_array[i].offset[1].y = 0;
			 piece_array[i].offset[2].x = 2;
			 piece_array[i].offset[2].y = 4;
			 piece_array[i].offset[3].x = 0;
			 piece_array[i].offset[3].y = 4;
			 break;
		case 3  :piece_array[i].block[0].x = 1;
			 piece_array[i].block[0].y = 3;
			 piece_array[i].block[1].x = 2;
			 piece_array[i].block[1].y = 2;
			 piece_array[i].block[2].x = 3;
			 piece_array[i].block[2].y = 1;
			 piece_array[i].offset[0].x = -2;
			 piece_array[i].offset[0].y = 0;
			 piece_array[i].offset[1].x = 4;
			 piece_array[i].offset[1].y = 0;
			 piece_array[i].offset[2].x = 2;
			 piece_array[i].offset[2].y = 4;
			 piece_array[i].offset[3].x = 0;
			 piece_array[i].offset[3].y = 4;

			 break;
		case 4  :piece_array[i].block[0].x = 1;
			 piece_array[i].block[0].y = 1;
			 piece_array[i].block[1].x = 2;
			 piece_array[i].block[1].y = 2;
			 piece_array[i].block[2].x = 3;
			 piece_array[i].block[2].y = 1;
			 piece_array[i].offset[0].x = 0;
			 piece_array[i].offset[0].y = 0;
			 piece_array[i].offset[1].x = 4;
			 piece_array[i].offset[1].y = 0;
			 piece_array[i].offset[2].x = 3;
			 piece_array[i].offset[2].y = 3;
			 piece_array[i].offset[3].x = -1;
			 piece_array[i].offset[3].y = 3;
 
			 break;
		case 5  :piece_array[i].block[0].x = 1;
			 piece_array[i].block[0].y = 1;
			 piece_array[i].block[1].x = 2;
			 piece_array[i].block[1].y = 2;
			 piece_array[i].block[2].x = 4;
			 piece_array[i].block[2].y = 2;
			 piece_array[i].offset[0].x = 0;
			 piece_array[i].offset[0].y = 0;
			 piece_array[i].offset[1].x = 2;
			 piece_array[i].offset[1].y = 0;
			 piece_array[i].offset[2].x = 5;
			 piece_array[i].offset[2].y = 3;
			 piece_array[i].offset[3].x = -1;
			 piece_array[i].offset[3].y = 3;


		}
		}/*for */

        fclose(fp);
        };

void make_piece_active(int piecenum)
	{
	extern Display *display;
	extern int active_piece;
	extern struct piece_rec piece_array[6];
	if (piece_array[piecenum].in_use)
		{
		XBell(display,100);
		/* make beep */
		return;
                }
	else
		{
                display_piece_on_piecewin(active_piece);
		display_piece_on_piecewin(piecenum);
                active_piece = piecenum;
	       }
        }
