#include <math.h>
#include <stdio.h>
#include  <stdlib.h>
#include "transfor.h"
#include "pieces.h"


void vector_copy(point new_vector, point old_vector)
	{
	int i;
	for (i=0;i<4;i++)
        	new_vector[i] = old_vector[i];
	}

void vector_mult(point old,trans_matrix operation,point the_new)
	{
	int i,j;
        point temp = {0,0,0,0};
	for (i=0;i<4;i++)
		for(j=0;j<4;j++)
		   temp[i] += old[j] * operation[i][j];
	temp[3] = 1;
	for (i=0;i<4;i++)
        	the_new[i] = temp[i];
	return;
	}

void vector_add(point to, point fr,point om)
	{
	to[0] = fr[0] + om[0];
	to[1] = fr[1] + om[1];
	to[2] = fr[2] + om[2];
	to[3] = 1.0;
	return;

	}


void identity(trans_matrix the_matrix)
	{
	int i,j;
	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			the_matrix[i][j] = (i == j);
	}

void rotate_x(trans_matrix the_matrix,double angle)
	{
	identity(the_matrix);
	the_matrix[1][1] = the_matrix[2][2] = cos(angle);
	the_matrix[2][1] = -(the_matrix[1][2] = sin(angle));

	}
void rotate_y(trans_matrix the_matrix,double angle)
	{
	identity(the_matrix);
	the_matrix[0][0] = the_matrix[2][2] = cos(angle);
	the_matrix[0][2] = -(the_matrix[2][0] = sin(angle));
	}
void rotate_z(trans_matrix the_matrix,double angle)
	{
	identity(the_matrix);
	the_matrix[0][0] = the_matrix[1][1] = cos(angle);
	the_matrix[0][1] = -(the_matrix[1][0] = sin(angle));
	}

void translate(trans_matrix the_matrix, point distances)
	{
	int i;
	identity(the_matrix);
	for(i=0;i<4;i++)
		the_matrix[i][3] = distances[i];
	}
void trans_x(trans_matrix the_matrix, int distance)
	{
	identity(the_matrix);
	the_matrix[0][3] = distance;
	}

void trans_y(trans_matrix the_matrix, int distance)
	{
	identity(the_matrix);
	the_matrix[1][3] = distance;
	}

void trans_z(trans_matrix the_matrix, int distance)
	{
	identity(the_matrix);
	the_matrix[2][3] = distance;
	}

void concat_trans_matricies(trans_matrix matrix1,
			    trans_matrix matrix2,
			    trans_matrix result)
	{
	int i,j,k;
	/* result can be the same matrix as matrix1 or matrix2
	so we need a temporary matrix */
	trans_matrix temp;
	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			{
			temp[i][j] = 0.0;
			for (k = 0; k < 4; k++)
				temp[i][j] +=matrix1[i][k] * matrix2[k][j];
			}
	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			result[i][j] = temp[i][j];
	}

void cross_product(point a, point b, point result)
	{
	result[0] = a[1] * b[2] - a[2] * b[1];
	result[1] = a[2] * b[0] - a[0] * b[2];
	result[2] = a[0] * b[1] - a[1] * b[0];
	}
double inner_product(point a, point b)
	{
	return (a[0] * b[0])+ (a[1] * b[1]) + (a[2] * b[2]);

	}
double vector_len(point v)
	{
	double temp;
	temp = (v[0] * v[0])+ (v[1] * v[1]) + (v[2] * v[2]);
	if (temp > 0.0001)
		return sqrt(temp);
	return 0.0;
	}

double twod_len(double p1x,double p1y, double p2x, double p2y)
	{
	double temp = (((p1x - p2x) * (p1x- p2x)) + ((p1y-p2y) * (p1y-p2y,2)));
	if (temp < 0.001)
		return 0.0;
	return (sqrt(temp));
	}


/* a point lies outside a triangle if you know one of its distances to points of
triangle longer than all edges of that triangle */

void vector_from_points(point p1, point p2, point vector)
	{
	vector[0] = p2[0] - p1[0];
	vector[1] = p2[1] - p1[1];
	vector[2] = p2[2] - p1[2];
	};

boolean point_in_triangle(point a, point tri, point ang, point le)
	{
	point vector1, vector2, vector3;
	vector_from_points(a,tri,vector1);
	vector_from_points(a,ang,vector2);
	vector_from_points(a,le,vector3);

	if (inner_product(vector1,vector2) < 0)
		return false;
	if (inner_product(vector2,vector3) < 0)
		return false;
	if (inner_product(vector3,vector1) < 0)
		return false;

/*	print_point(a);
	printf("lies in triangle\n");
	print_point(tri);
	print_point(ang);
	print_point(le);
*/	return true;
	}

double point_z_location(point a,point tri,point ang,point le)
	{
	/* construct a plane from triangle */
	point normal;
	point vector1,vector2;
	vector_from_points(ang,tri,vector1);
	vector_from_points(ang,le,vector2);
	cross_product(vector1,vector2,normal);
	/* calculate intersection point */
	if (normal[2] < 0.0001)
		{
		printf("error in normal");
		exit(1);
		};
	printf("ok\n");
	return ( ( ( (normal[0] * (a[0] - ang[0])) +
		     (normal[1] * (a[1] - ang[1]))
		   ) /normal[2]
		 ) - ang[2]
	       );
	}
/*
void scale(double input[4], double ratiox, double ratioy, double ratioz)
	{
	double output[4] = {0.0,0.0,0.0,0.0} ;

	int i,j;

	double matrix[4][4] = { {1.0,0.0,0.0,0.0},
				{0.0,1.0,0.0,0.0},
				{0.0,0.0,1.0,0.0},
				{0.0,0.0,0.0,1.0}};


	matrix[0][0] = dx;
	matrix[1][1] = dy;
	matrix[2][2] = dz;

	for(i=0;i<4;i++)
		for(j=0;j<4;j++)
			output[i] += matrix[i][j] * input[i];
	return output;
	}

*/
/*
double *shear(double input[4], double ratiox, double ratioy, double ratioz)
	{
	double output[4] = {0.0,0.0,0.0,0.0} ;

	int i,j;

	double matrix[4][4] = { {1.0,0.0,0.0,0.0},
				{0.0,1.0,0.0,0.0},
				{0.0,0.0,1.0,0.0},
				{0.0,0.0,0.0,1.0}};


	matrix[0][0] = dx;
	matrix[1][1] = dy;
	matrix[2][2] = dz;

	for(i=0;i<4;i++)
		for(j=0;j<4;j++)
			output[i] += matrix[i][j] * input[i];
	return output;
	}

*/
